#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
using System.Reflection;
using System.Windows.Forms;
#endregion


namespace NinjaTrader.Indicator
{
    /// <summary>
    /// 
    /// </summary>
	/// 
	/// 
	public class BubbleUtils {
		
		private  static int getXByBarIdxDouble(double barDouble, ChartControl chart) {
			return (int)typeof(ChartControl).GetMethod("GetXByBarIdxDouble",BindingFlags.NonPublic|BindingFlags.Instance).Invoke(chart,new Object [] {barDouble});
	
		}
		
		public  static Point getEnd(ChartLine line, IndicatorBase  indicator) {
			ChartControl chart = indicator.ChartControl;
			if (line==null) return new Point(0,0);
			int x,y;
			if (chart.EquidistantBars==true) {
			
				double end_bar_double = (double)typeof(ChartLine).GetProperty("endBarDouble",BindingFlags.NonPublic | BindingFlags.Instance).GetValue(line,null);
				x = getXByBarIdxDouble(end_bar_double,chart);
				y = chart.GetYByValue(indicator,line.EndY);
				return new Point(x,y);
			}
			x = chart.GetXByTime(line.EndTime); 
			y = chart.GetYByValue(indicator,line.EndY);
			return new Point(x,y);
		}	
		public  static Point getStart (ChartLine line,IndicatorBase indicator) {
			ChartControl chart = indicator.ChartControl;
			if (line==null) return new Point(0,0);
			int x,y;
			if (chart.EquidistantBars==true) {
				double start_bar_double = (double)typeof(ChartObject).GetProperty("startBarDouble",BindingFlags.NonPublic | BindingFlags.Instance).GetValue(line,null);
				x = getXByBarIdxDouble(start_bar_double,chart);
				y = chart.GetYByValue(indicator,line.StartY);
				return new Point(x,y);
			}
			
			x = chart.GetXByTime(line.StartTime);
			y=  chart.GetYByValue(indicator,line.StartY);
			return new Point(x,y);
		}	
	}
	
	
	
    [Description("")]
    public class Bubble : Indicator
    {
        #region Variables
        // Wizard generated variables
        Color line_color = Color.LimeGreen;
		int line_width = 1;
        // User defined variables (add any user defined variables below)
        #endregion


		
 public struct ColorRGB

      {

            public byte R;

            public byte G;

            public byte B;

		
            public ColorRGB(Color value)

            {

                  this.R = value.R;

                  this.G = value.G;

                  this.B = value.B;

            }

            public static implicit operator Color(ColorRGB rgb)

            {

                  Color c = Color.FromArgb(rgb.R,rgb.G,rgb.B);

                  return c;

            }

            public static explicit operator ColorRGB(Color c)

            {

                  return new ColorRGB(c);

            }

      }
	
	  // Given H,S,L in range of 0-1

      // Returns a Color (RGB struct) in range of 0-255

      public static ColorRGB HSL2RGB(double h, double sl, double l)

      {

            double v;

            double r,g,b;

 

            r = l;   // default to gray

            g = l;

            b = l;

            v = (l <= 0.5) ? (l * (1.0 + sl)) : (l + sl - l * sl);

            if (v > 0)

            {

                  double m;

                  double sv;

                  int sextant;

                  double fract, vsf, mid1, mid2;

 

                  m = l + l - v;

                  sv = (v - m ) / v;

                  h *= 6.0;

                  sextant = (int)h;

                  fract = h - sextant;

                  vsf = v * sv * fract;

                  mid1 = m + vsf;

                  mid2 = v - vsf;

                  switch (sextant)

                  {

                        case 0:

                              r = v;

                              g = mid1;

                              b = m;

                              break;

                        case 1:

                              r = mid2;

                              g = v;

                              b = m;

                              break;

                        case 2:

                              r = m;

                              g = v;

                              b = mid1;

                              break;

                        case 3:

                              r = m;

                              g = mid2;

                              b = v;

                              break;

                        case 4:

                              r = mid1;

                              g = m;

                              b = v;

                              break;

                        case 5:

                              r = v;

                              g = m;

                              b = mid2;

                              break;

                  }

            }

            ColorRGB rgb;

            rgb.R = Convert.ToByte(r * 255.0f);

            rgb.G = Convert.ToByte(g * 255.0f);

            rgb.B = Convert.ToByte(b * 255.0f);

            return rgb;

      }
	
	
	
	

      // Given a Color (RGB Struct) in range of 0-255

      // Return H,S,L in range of 0-1

      public static void      RGB2HSL (ColorRGB rgb, out double h, out double s, out double l)

      {

            double r = rgb.R/255.0;

            double g = rgb.G/255.0;

            double b = rgb.B/255.0;

            double v;

            double m;

            double vm;

            double r2, g2, b2;

 

            h = 0; // default to black

            s = 0;

            l = 0;

            v = Math.Max(r,g);

            v = Math.Max(v,b);

            m = Math.Min(r,g);

            m = Math.Min(m,b);

            l = (m + v) / 2.0;

            if (l <= 0.0)

            {

                  return;

            }

            vm = v - m;

            s = vm;

            if (s > 0.0)

            {

                  s /= (l <= 0.5) ? (v + m ) : (2.0 - v - m) ;

            }

            else

            {

                  return;

            }

            r2 = (v - r) / vm;

            g2 = (v - g) / vm;

            b2 = (v - b) / vm;

            if (r == v)

            {

                  h = (g == m ? 5.0 + b2 : 1.0 - g2);

            }

            else if (g == v)

            {

                  h = (b == m ? 1.0 + r2 : 3.0 - b2);

            }

            else

            {

                  h = (r == m ? 3.0 + g2 : 5.0 - r2);

            }

            h /= 6.0;

      }
	
	
        protected override void Initialize()
        {
          
            Overlay				= true;
			CalculateOnBarClose = false;
			ChartOnly = true;	
        }
	
		public ChartLine[] getLines (Color color, float thickness) {
			int length = DrawObjects.Count;
			if (DrawObjects.Count==0) return null;
			int number_of_lines = 0;
			foreach (IDrawObject obj in DrawObjects) {
				if (obj.DrawType == DrawType.Line && ((ChartLine)obj).Pen.Color==color && ((ChartLine)obj).Pen.Width == thickness) 
					++number_of_lines;
			}
			if (number_of_lines==0) return null;
			ChartLine[] objects = new ChartLine[number_of_lines];
			int index=0;
			foreach (IDrawObject obj in DrawObjects) {
				if (obj.DrawType ==DrawType.Line && ((ChartLine)obj).Pen.Color==color && ((ChartLine)obj).Pen.Width ==thickness) {
					objects[index] = (ChartLine)obj; ++index;
				}
			}
			return objects;			
		}
		
		public override void Plot(Graphics g , Rectangle bounds, double min, double max) {
			ChartLine[] lines = getLines(line_color,line_width);
			g.SmoothingMode = SmoothingMode.AntiAlias;
			if (lines!=null) {
		
				Color red_color = HSL2RGB(0.0,1,.36);
				red_color = Color.FromArgb(50,red_color.R,red_color.G,red_color.B);
				Color light_red = HSL2RGB(0,1,0.5);
				Color green_color = HSL2RGB(120/360.0,0.61,.36);
				green_color = Color.FromArgb(50,green_color.R,green_color.G,green_color.B);
				Color light_green = HSL2RGB(120/360.0,0.61,0.5);
				
				Color blue_color = HSL2RGB(247/360.0,1,0.45);
				blue_color = Color.FromArgb(50,blue_color.R,blue_color.G,blue_color.B);
				Color light_blue = HSL2RGB(247/360.0,1,0.64);
				
                Brush red_brush = new SolidBrush(red_color);
				Brush light_red_brush = new SolidBrush(light_red);
				
			    Brush green_brush = new SolidBrush(green_color);
				Brush light_green_brush = new SolidBrush(light_green);
				
				Brush blue_brush = new SolidBrush(blue_color);
				Brush light_blue_brush = new SolidBrush(light_blue);
				
				Pen red_pen = new Pen(light_red_brush);
				red_pen.Width = 2F;
				Pen green_pen = new Pen(light_green_brush);
				green_pen.Width = 2F;
				Pen blue_pen = new Pen(light_blue_brush);
				blue_pen.Width = 2;
				
				foreach (ChartLine line in lines) {
					 //draw two circles per line
					Point start = BubbleUtils.getStart(line,this);
					Point end = BubbleUtils.getEnd(line,this);

					
						double [] red = {Math.Sqrt(2),1.5};
					double [] green  = {2,Math.Sqrt(5)};
						
					double radius = Math.Sqrt(Math.Pow(start.Y-end.Y,2)+Math.Pow(start.X-end.X,2));
					double delta_x = end.X-start.X;
					double delta_y = end.Y-start.Y;
					double green_radius = (green[1]/Math.Sqrt(10)*radius- green[0]/Math.Sqrt(10)*radius)/2.0;
					double red_radius = (red[1]/Math.Sqrt(10)*radius-red[0]/Math.Sqrt(10)*radius)/2.0;
					double blue_radius = red_radius/2.0;
				
					double green_ratio = (green[0]+green[1])/(2*Math.Sqrt(10));
					double red_ratio = ((red[1])/(Math.Sqrt(10)) *radius + red_radius)/radius;
					double blue_ratio = 1/(Math.Sqrt(10));
					
 					double []green_center = {start.X+green_ratio * delta_x, start.Y+green_ratio * delta_y};
					double []red_center = {start.X+red_ratio*delta_x, start.Y+red_ratio*delta_y};
					double [] blue_center = {start.X + blue_ratio*delta_x,start.Y+blue_ratio*delta_y};
						blue_pen.DashStyle = DashStyle.Solid;
					red_pen.DashStyle = DashStyle.Solid;
					green_pen.DashStyle = DashStyle.Solid;
					blue_pen.Width = 2;
					red_pen.Width = 2;
					green_pen.Width = 2;
			
					g.FillEllipse(green_brush,(float)(green_center[0]-green_radius), (float)(green_center[1]-green_radius),(float)(2*green_radius),
					                  (float)(2*green_radius));
					g.DrawEllipse(green_pen,(float)(green_center[0]-green_radius), (float)(green_center[1]-green_radius),(float)(2*green_radius),
					                  (float)(2*green_radius));
					
					radius = Math.Sqrt(Math.Pow(start.X-green_center[0],2)+Math.Pow(start.Y-green_center[1],2));
					green_pen.Width = 1;
					green_pen.DashStyle = DashStyle.Dot;
					g.DrawArc(green_pen,(float)(start.X-radius),(float)(start.Y-radius),(float)(2*radius),(float)(2*radius),0,360);
					
					
					g.FillEllipse(red_brush,(float)(red_center[0]-red_radius),(float)(red_center[1]-red_radius),(float)(2*red_radius),
					                  (float)(2*red_radius));
					g.DrawEllipse(red_pen,(float)(red_center[0]-red_radius),(float)(red_center[1]-red_radius),(float)(2*red_radius),
					                  (float)(2*red_radius));
					radius = Math.Sqrt(Math.Pow(start.X-red_center[0],2)+Math.Pow(start.Y-red_center[1],2));
					red_pen.Width = 1;
					red_pen.DashStyle = DashStyle.Dot;
					g.DrawArc(red_pen,(float)(start.X-radius),(float)(start.Y-radius),(float)(2*radius),(float)(2*radius),0,360);
					
					g.FillEllipse(blue_brush,(float)(blue_center[0]-blue_radius),(float)(blue_center[1]-blue_radius),(float)(2*blue_radius),
					                  (float)(2*blue_radius));
					g.DrawEllipse(blue_pen,(float)(blue_center[0]-blue_radius),(float)(blue_center[1]-blue_radius),(float)(2*blue_radius),
					                  (float)(2*blue_radius));
					radius = Math.Sqrt(Math.Pow(start.X-blue_center[0],2)+Math.Pow(start.Y-blue_center[1],2));
					blue_pen.Width = 1;
					blue_pen.DashStyle = DashStyle.Dot;
					g.DrawArc(blue_pen,(float)(start.X-radius),(float)(start.Y-radius),(float)(2*radius),(float)(2*radius),0,360);
					
				
					
					
				}
				
				
				
				
			}
		}
		
        protected override void OnBarUpdate()
        {
			
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Plot0
        {
            get { return Values[0]; }
        }

       
		[XmlIgnore()]
		[Description("Line Width")]
        [GridCategory("Parameters")]
		public int lineWidth {
			get {return line_width;}
			set{line_width = value;}}
		
		
		[XmlIgnore()]
		[Description("Line Color")]
        [GridCategory("Parameters")]
		public Color lineColor 
		{
			get {return line_color;}
			set {line_color=value;}
		}
		
		[Browsable(false)]
		public string lineColorSerializable
		{
     		get { return NinjaTrader.Gui.Design.SerializableColor.ToString(line_color); }
     		set { line_color = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private Bubble[] cacheBubble = null;

        private static Bubble checkBubble = new Bubble();

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public Bubble Bubble(Color lineColor, int lineWidth)
        {
            return Bubble(Input, lineColor, lineWidth);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public Bubble Bubble(Data.IDataSeries input, Color lineColor, int lineWidth)
        {
            if (cacheBubble != null)
                for (int idx = 0; idx < cacheBubble.Length; idx++)
                    if (cacheBubble[idx].lineColor == lineColor && cacheBubble[idx].lineWidth == lineWidth && cacheBubble[idx].EqualsInput(input))
                        return cacheBubble[idx];

            lock (checkBubble)
            {
                checkBubble.lineColor = lineColor;
                lineColor = checkBubble.lineColor;
                checkBubble.lineWidth = lineWidth;
                lineWidth = checkBubble.lineWidth;

                if (cacheBubble != null)
                    for (int idx = 0; idx < cacheBubble.Length; idx++)
                        if (cacheBubble[idx].lineColor == lineColor && cacheBubble[idx].lineWidth == lineWidth && cacheBubble[idx].EqualsInput(input))
                            return cacheBubble[idx];

                Bubble indicator = new Bubble();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.lineColor = lineColor;
                indicator.lineWidth = lineWidth;
                Indicators.Add(indicator);
                indicator.SetUp();

                Bubble[] tmp = new Bubble[cacheBubble == null ? 1 : cacheBubble.Length + 1];
                if (cacheBubble != null)
                    cacheBubble.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheBubble = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.Bubble Bubble(Color lineColor, int lineWidth)
        {
            return _indicator.Bubble(Input, lineColor, lineWidth);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public Indicator.Bubble Bubble(Data.IDataSeries input, Color lineColor, int lineWidth)
        {
            return _indicator.Bubble(input, lineColor, lineWidth);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.Bubble Bubble(Color lineColor, int lineWidth)
        {
            return _indicator.Bubble(Input, lineColor, lineWidth);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public Indicator.Bubble Bubble(Data.IDataSeries input, Color lineColor, int lineWidth)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.Bubble(input, lineColor, lineWidth);
        }
    }
}
#endregion
